/*
    Copyright 2009 Sebastian Kłęk, Piotr Domagala and Jakub Mielczarek
    This file is part of the Filtertool.

    Filtertool is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <vector>
#include <map>
#include <algorithm>
#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <ctime>
#include <cmath>
#include "DataHandler.h"

using namespace std;

/*! \brief DataOperations: class to handle data and perform operations on them
	This class inherits DataHandler (interface). This construction allows to dynamicly select the temlate argument.
	\sa DataHandler
	\author Sebastian Kłęk, Piotr Domagala and Jakub Mielczarek
 */
template <typename T> class DataOperations: public DataHandler {
public:

/*! This is the class constructor.
	It takes no arguments and it's basicly doing nothing exept setting default values to some variables.
 */
	DataOperations(){
		isJoined = false;
		isMerged = false;
		tabulator = false;
		columns = false;
		tolerance = 0;
		precision = 0;
		blank = "--";
		commentMode = 0;
	};

/*! This function swaps two columns in selected file.
	\param command is a command line argument of -s (--swap) option
 */
	virtual void swap(string command);

/*! Function to collect input file names.
	\param names is a list of files separated by ','
 */
	virtual void parseFiles(string names );

/*! This function loads data to memory.
	If input files have been set, this function will open each of them
	and read content. Otherwise it will start reading standard input.
	The EOF symbol ends reading.
 */
	virtual void load();

/*! This function joins all input to one output.
   Actually it works only when two or more files were specified on the input
 */
	virtual void join() {
		isJoined = true;
	}

	//! This function merge all input data.
	virtual void merge();

	//! This function makes all calculations to be made by columns in stead of by lines.
	virtual void col(){
		columns = true;
	}

	//! This function switch spaces into tabulators.
	virtual void tab() {
		tabulator = true;
	}

/*! This function is called on the end, when we want to print all.
	Function will decide where to save the data one file, many files
	or print to standard output.
 */
	virtual void printOut();

/*! Setter for command that started the program.
	\param cmd is a string containing command that started the program.
 */
	virtual void setCommand(string cmd) {
		command = cmd;
	}

/*! Setter for blanks.
	\param arg is a string containing symbols putted where invalid value is found .
 */
	virtual void blanks(string arg) {
		blank = arg;
	}

/*! Setter for tolerance used in comparations made by filter function.
	\param t value of tolerance.
 */
	virtual void tol(double t){
		tolerance = (T) t;
	}

/*! Setter for printing precision.
	\param p precision (number of digits after '.').
 */
	virtual void prec(int p){
		precision = p;
	}

/*! This function parses the filtr option.
	\param arg is a command line argument of -f (--filtr) option.
 */
	virtual void setFilter(string arg);

/*! Sets the output file names.
	\param names is a list of output files specified by -o (--output-files) option and separated by ','.
 */
	virtual void setOutputFiles(string names);

/*! This function parses the group option and accordingly to it groups data.
	\param arg is a command line argument of -g (--group) option.
 */
	virtual void group(string arg);

/*! This function parses the comments option and accordingly to it sets level of coping comments from input to output.
	\param arg is a command line argument of --comments option.
 */
	virtual void comments(string arg);

/*! This function parses the calculate option and accordingly to it performs selected operations.
	\param arg is a command line argument of -c (--calculate) option.
 */
	virtual void calculate(string arg);

/*! This function parses the print option and accordingly to it prints only selected data.
	\param arg is a command line argument of -p (--print) option.
 */
	virtual void print(string arg);

private:
	/*! \brief Class for storing values
		It can store number or blank wich allows to display missing data.
		\author Sebstian Kłęk
	 */
	class Value {
	public:
		//! If true this value is number. False indicates that there is error in data file or missing data.
		bool isNumber;

		//! Stored number (0 if blank)
		T val;

		/*! Class constructor.
			\param v number to store
		*/
		Value (T v) : isNumber(true), val(v) {};

		//! Class constructor. New value is blank.
		Value () : isNumber(false), val(0) {};
	};

	//! variable used in filters functions to store information wich column is with condition.
	size_t col_cond;

	//! if true input files are treated as one.
	bool isJoined;

	//! if true input files are merged.
	bool isMerged;

	//! if true tabulators are used to separate values
	bool tabulator;

	//! if true all calculations are made on columns
	bool columns;

	/*! variable used in filters functions to store information about condidions
		(0 - means only one condition, a - means and, o - means or).
	 */
	char c;

	//! stores size of tolerance used in filters functions (0 - default)
	T tolerance;

	/*! stores number of digits to display after comma
		(0 - default: means there wasn't --prec option in command, 6 digits are used)
	 */
	int precision;

	//! stores string that will be put where data is missing
	string blank;

	//! stores information about comments mode (-1 off, 0 default, 1 one, 2 all)
	int commentMode;

	//! stores names of input files
	vector<string> file_names;

	//! stores names of output files
	vector<string> file_out;

	//! main data structure, stores all values, it's 3D vector: all_values[files][lines][columns]
	vector< vector< vector<Value> > > all_values;

	//! 2D vector for storing comments
	vector< vector<string> > all_comments;

	//! vector of maps for storing parameters fuond in comments
	vector< map<string, double> > all_param;

	//! temporary 2D vector for storing values
	vector< vector<Value> > tab_tmp;

	//! temporary vector for storing values
	vector<Value> vall_tmp;

	//! vector of numbers used in filters functions for comparations
	vector<double> conditions;

	//! vector of condidions used in filters functions
	vector<string> kind_of_cond;

	//! vector for storing numbers of columns that will be printed, used in filters functions
	vector<size_t> out;

	//! 2D vector for storing current dimmention of data, used in calculations functions
	vector< vector <size_t> > dim;

	//! stores command that started the program
	string command;

	//! Private copy constructor prevents from creating two same objects
	DataOperations(DataOperations &) {}

	//! Function to print data on standard output
	void printToStd();

	//! Function to print data to one file
	void printToFile();

	//! Function to print data to many files
	void printToFiles();

/*! This function starts filtering data with condidions given in setFilter
  \sa setFilter
*/
	size_t startFiltering();

/*! Funtion to calculate mean.
	It can works on columns or rows.
 */
	void mean();

/*! Funtion to count values.
	It can works on columns or rows.
 */
	void count();

/*! Funtion to calculate sum of values.
	It can works on columns or rows.
 */
	void sum();

/*! Funtion to calculate median.
	It can works on columns or rows.
 */
	void median();

/*! Funtion to calculate standard deviation.
	It can works on columns or rows.
 */
	void deviation();

/*! Funtion to search for missing data and fill in blanks when needed.
	This is made because the same number of columns in every line in needed.
 */
	void fillInBlanks();

	//! This funtion inserts comments before data.
	void commOut(ostream* out);

	//! This funtion inserts comments before data of each file.
	void commIn(ostream* out, int i);

	//! This funtion parses comments, finds all parameters and put it in map.
	void parseComments();
};

template <typename T> void DataOperations<T>::fillInBlanks(){
	if (isJoined){
		size_t max_col = 0;
		for(size_t i = 0; i < all_values.size(); i++) {
			for(size_t j = 0; j < all_values[i].size(); j++) {
				if (all_values[i][j].size() > max_col) max_col = all_values[i][j].size();
			}

		}
		for(size_t i = 0; i < all_values.size(); i++) {
			for(size_t j = 0; j < all_values[i].size(); j++) {
				if (all_values[i][j].size() < max_col){
					for (size_t k = all_values[i][j].size(); k < max_col; k++) {
						all_values[i][j].push_back(Value());
					}
				}
			}
		}
	}
	else {
		for(size_t i = 0; i < all_values.size(); i++) {
			size_t max_col = 0;
			for(size_t j = 0; j < all_values[i].size(); j++) {
				if (all_values[i][j].size() > max_col) max_col = all_values[i][j].size();
			}
			for(size_t j = 0; j < all_values[i].size(); j++) {
				if (all_values[i][j].size() < max_col){
					for (size_t k = all_values[i][j].size(); k < max_col; k++) {
						all_values[i][j].push_back(Value());
					}
				}
			}
		}
	}
}

template <typename T> void DataOperations<T>::parseComments() {
	size_t a;
	size_t b;
	size_t e;
	string tmp;
	string name;
	double value;
	const char* white = " \t";
	all_param.resize(all_comments.size());
	for (size_t i = 0; i < all_comments.size(); i++){
		for (size_t j = 0; j < all_comments[i].size(); j++){
			tmp = all_comments[i][j];
			a = tmp.find_first_of("=");
			name = tmp.substr(0, a);
			while (a != string::npos){
				if ( tmp[a-1] == ' ' || tmp[a-1] == '\t' ){
					e = name.find_last_not_of(white);
				}
				else {
					e = a-1;
				}
				b = name.find_last_of(white, e);
				if (b != string::npos) {
					name = name.substr(b+1, e-b);
				}
				else {
					name = name.substr(0, e+1);
				}
				tmp = tmp.substr(a+1);
				value = atof(tmp.c_str());
				if(value != 0 || (value == 0 && (tmp == "0" || tmp.find("0.0") == 0))) {
					all_param[i][name] = value;
				}
				else {
					cerr << "Invalid value of parameter " << name << " in data file: " << i+1 << endl;
				}
				a = tmp.find_first_of("=");
				name = tmp.substr(0, a);
			}
		}
	}

}

template <typename T> void DataOperations<T>::parseFiles(string names) {
	char delim = ',';
	string name;
	istringstream iss(names);
	while (getline(iss, name, delim)) {
		file_names.push_back(name);
	}
	load();
	parseComments();
}

template <typename T> void DataOperations<T>::load() {
	vector< vector<Value> > tab_tmp;
	vector<string> com_tmp;
	vector<Value> val_tmp;
	string s, line;
	T liczba;
	size_t found, found1;
	const char* white = " \t\r\n";
	size_t ile = file_names.size();
	if(file_names.size() == 0) ile = 1;
	for(size_t i = 0; i < ile; i++) {
		istream* in;
		ifstream infile;
		if(file_names.size() != 0) {
			infile.open(file_names[i].c_str());
			in = &infile;
		}
		else {
			in = &cin;
		}
		while(getline(*in, line)) {
			found = line.find_first_not_of(white);
			if (found != string::npos && line[found] == '#') {
				com_tmp.push_back(line.substr(found));
			}
			else if(found != string::npos && line[found] != '#') {
				line = line+"\r\n";
				do {
					found1 = line.find_first_of(white, found);
					if(found1 != string::npos) s = line.substr(found, found1-found);
					found = line.find_first_not_of(white, found1);
					liczba = (T) atof(s.c_str());
					if(liczba != 0 || (liczba == 0 && (s == "0" || s.find("0.0") == 0))) {
						val_tmp.push_back(Value(liczba));
					}
					else {
						cerr << "Invalid value detected in data file: " << i+1 << endl;
						val_tmp.push_back(Value());
					}
				} while(found != string::npos && found1 != string::npos);
				if(!val_tmp.empty()) {
					tab_tmp.push_back(val_tmp);
					val_tmp.clear();
				}
			}
		}
		if(!tab_tmp.empty()) {
			all_values.push_back(tab_tmp);
			tab_tmp.clear();
		}
		if(!com_tmp.empty()) {
			all_comments.push_back(com_tmp);
			com_tmp.clear();
		} else {
			com_tmp.push_back("\r\n");
			all_comments.push_back(com_tmp);
			com_tmp.clear();
		}
	}
	fillInBlanks();
}

template <typename T> void DataOperations<T>::swap(string arg) {
	size_t n, m, file;
	n = atoi(arg.c_str());
	size_t found = arg.find_first_of(',');
	size_t found2 = arg.find_last_of(',');
	m = atoi(arg.substr(found+1, found2).c_str());
	file = atoi(arg.substr(found2+1).c_str());
	size_t begin;
	size_t end;
	Value tmp;
	n--; m--; file--;
	if(isJoined || found == found2) {
		begin = 0;
		end = all_values.size();
		file = 0;
	}
	else {
		begin = file;
		end = file+1;
	}
	for(size_t i = begin; i < end; i++) {
		if(n >= all_values[i][0].size() || m >= all_values[i][0].size() || n == m || file > all_values.size() || begin >= all_values.size()) {
			cerr << "Wrong swap argument!" << endl;
			exit(1);
		}
	}
	for( /* begin  */; begin < end; begin++) {
		for(size_t j = 0; j < all_values[begin].size(); j++) {
			tmp = all_values[begin][j][n];
			all_values[begin][j][n] = all_values[begin][j][m];
			all_values[begin][j][m] = tmp;
		}
	}
}

template <typename T> void DataOperations<T>::mean() {
	T tmp = 0;
	int l = 0;
	vector<Value> tmp_vec;
	vector<int> x;
	if (columns){
		if (isJoined){
			if (dim[0].size() > 0){
				tmp_vec.resize(dim[0][0]);
				x.resize(dim[0][0]);
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val += all_values[i][j][k].val;
								tmp_vec[k].isNumber = true;
								x[k]++;
							}
						}
					}
				}
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (x[k] > 0 && tmp_vec[k].isNumber){
								tmp_vec[k].val = (T) (tmp_vec[k].val / (double)x[k]);
								x[k] = -1;
							}
						}
					}
				}
				all_values[dim.size()-1].push_back(tmp_vec);
			}
		}
		else {
			for(size_t i = 0; i < dim.size(); i++) {
				if (dim[0].size() > 0){
					tmp_vec.clear();
					tmp_vec.resize(dim[i][0]);
					x.clear();
					x.resize(dim[i][0]);
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val += all_values[i][j][k].val;
								tmp_vec[k].isNumber = true;
								x[k]++;
							}
						}
					}
					for(size_t k = 0; k < dim[i][0]; k++) {
						if (x[k] > 0 && tmp_vec[k].isNumber){
							tmp_vec[k].val = (T) (tmp_vec[k].val / (double)x[k]);
							x[k] = -1;
						}
					}
					all_values[i].push_back(tmp_vec);
				}
			}
		}
	}
	else {
		for(size_t i = 0; i < dim.size(); i++) {
			for(size_t j = 0; j < dim[i].size(); j++) {
				for(size_t k = 0; k < dim[i][j]; k++) {
					if (all_values[i][j][k].isNumber){
						tmp += all_values[i][j][k].val;
						l++;
					}
				}
				tmp = (T) (tmp/double(l));
				all_values[i][j].push_back(Value(tmp));
				tmp = 0 ;
				l = 0;
			}
		}
	}
}

template <typename T> void DataOperations<T>::count() {
	size_t tmp = 0;
	vector<Value> tmp_vec;
	if (columns){
		if (isJoined){
			if (dim[0].size() > 0){
				tmp_vec.resize(dim[0][0]);
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val++;
								tmp_vec[k].isNumber = true;
							}
						}
					}
				}
			}
			all_values[dim.size()-1].push_back(tmp_vec);
		}
		else {
			for(size_t i = 0; i < dim.size(); i++) {
				if (dim[0].size() > 0){
					tmp_vec.clear();
					tmp_vec.resize(dim[i][0]);
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val++;
								tmp_vec[k].isNumber = true;
							}
						}
					}
					all_values[i].push_back(tmp_vec);
				}
			}
		}
	}
	else {
		for(size_t i = 0; i < dim.size(); i++) {
			for(size_t j = 0; j < dim[i].size(); j++) {
				for(size_t k = 0; k < dim[i][j]; k++) {
					if (all_values[i][j][k].isNumber){
						tmp++;
					}
				}
				all_values[i][j].push_back(Value(tmp));
				tmp = 0 ;
			}
		}
	}
}

template <typename T> void DataOperations<T>::sum() {
	T tmp = 0;
	vector<Value> tmp_vec;
	if (columns){
		if (isJoined){
			if (dim[0].size() > 0){
				tmp_vec.resize(dim[0][0]);
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val += all_values[i][j][k].val;
								tmp_vec[k].isNumber = true;
							}
						}
					}
				}
				all_values[dim.size()-1].push_back(tmp_vec);
			}
		}
		else {
			for(size_t i = 0; i < dim.size(); i++) {
				if (dim[0].size() > 0){
					tmp_vec.clear();
					tmp_vec.resize(dim[i][0]);
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val += all_values[i][j][k].val;
								tmp_vec[k].isNumber = true;
							}
						}
					}
					all_values[i].push_back(tmp_vec);
				}
			}
		}
	}
	else {
		for(size_t i = 0; i < dim.size(); i++) {
			for(size_t j = 0; j < dim[i].size(); j++) {
				for(size_t k = 0; k < dim[i][j]; k++) {
					if (all_values[i][j][k].isNumber){
						tmp += all_values[i][j][k].val;
					}
				}
				all_values[i][j].push_back(Value(tmp));
				tmp = 0 ;
			}
		}
	}
}

template <typename T> void DataOperations<T>::deviation() {
	T tmp = 0;
	T tmp2 = 0;
	int l = 0;
	vector<Value> tmp_vec;
	vector<Value> tmp_vec2;
	vector<int> x;
	if (columns){
		if (isJoined){
			if (dim[0].size() > 0){
				tmp_vec.resize(dim[0][0]);
				tmp_vec2.resize(dim[0][0]);
				x.resize(dim[0][0]);
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val += all_values[i][j][k].val;
								tmp_vec[k].isNumber = true;
								tmp_vec2[k].val += all_values[i][j][k].val*all_values[i][j][k].val;
								tmp_vec2[k].isNumber = true;
								x[k]++;
							}
						}
					}
				}
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (x[k] > 0 && tmp_vec[k].isNumber){
								tmp_vec[k].val = (T) (tmp_vec[k].val / (double)x[k]);
								tmp_vec[k].val *= tmp_vec[k].val;
								tmp_vec2[k].val = (T) (tmp_vec2[k].val / (double)x[k]);
								tmp_vec[k].val = tmp_vec2[k].val-tmp_vec[k].val;
								tmp_vec[k].val = (T) ( tmp_vec[k].val*x[k]/(double)(x[k]-1) );
								tmp_vec[k].val = (T) sqrt(tmp_vec[k].val);
								x[k] = -1;
							}
						}
					}
				}
				all_values[dim.size()-1].push_back(tmp_vec);
			}
		}
		else {
			for(size_t i = 0; i < dim.size(); i++) {
				if (dim[0].size() > 0){
					tmp_vec.clear();
					tmp_vec.resize(dim[i][0]);
					tmp_vec2.clear();
					tmp_vec2.resize(dim[i][0]);
					x.clear();
					x.resize(dim[0][0]);
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								tmp_vec[k].val += all_values[i][j][k].val;
								tmp_vec[k].isNumber = true;
								tmp_vec2[k].val += all_values[i][j][k].val*all_values[i][j][k].val;
								tmp_vec2[k].isNumber = true;
								x[k]++;
							}
						}
					}
					for(size_t k = 0; k < dim[i][0]; k++) {
						if (x[k] > 0 && tmp_vec[k].isNumber){
							tmp_vec[k].val = (T) (tmp_vec[k].val / (double)x[k]);
							tmp_vec[k].val *= tmp_vec[k].val;
							tmp_vec2[k].val = (T) (tmp_vec2[k].val / (double)x[k]);
							tmp_vec[k].val = tmp_vec2[k].val - tmp_vec[k].val;
							tmp_vec[k].val = (T) ( tmp_vec[k].val*x[k] / (double)(x[k]-1) );
							tmp_vec[k].val = (T) sqrt(tmp_vec[k].val);
							x[k] = -1;
						}
					}
					all_values[i].push_back(tmp_vec);
				}
			}
		}
	}
	else {
		for(size_t i = 0; i < dim.size(); i++) {
			for(size_t j = 0; j < dim[i].size(); j++) {
				for(size_t k = 0; k < dim[i][j]; k++) {
					if (all_values[i][j][k].isNumber){
						tmp += all_values[i][j][k].val;
						tmp2 += all_values[i][j][k].val*all_values[i][j][k].val;
						l++;
					}
				}
				tmp = (T) (tmp/double(l));
				tmp *= tmp;
				tmp2 = (T) (tmp2/double(l));
				tmp = tmp2-tmp;
				tmp = (T) ( tmp*l/(double(l)-1) );
				tmp = (T) sqrt(tmp);
				all_values[i][j].push_back(Value(tmp));
				tmp = 0 ;
				tmp2 = 0 ;
				l = 0;
			}
		}
	}
}

template <typename T> void DataOperations<T>::median() {
	vector<Value> tmp_vec;
	vector<T> x;
	vector< vector<T> > y;
	T tmp;
	if (columns){
		if (isJoined){
			if (dim[0].size() > 0){
				tmp_vec.resize(dim[0][0]);
				y.resize(dim[0][0]);
				for(size_t i = 0; i < dim.size(); i++) {
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][j]; k++) {
							if (all_values[i][j][k].isNumber){
								y[k].push_back(all_values[i][j][k].val);
							}
						}
					}
				}
				for(size_t k = 0; k < dim[0][0]; k++) {
					sort(y[k].begin(), y[k].end());
				}
				for(size_t k = 0; k < dim[0][0]; k++) {
					if (y[k].size()%2 == 1){
						tmp_vec[k].val = y[k][y[k].size()/2];
						tmp_vec[k].isNumber = true;
					}
					else {
						tmp_vec[k].val = (y[k][(y[k].size()/2)-1] + y[k][y[k].size()/2])/2;
						tmp_vec[k].isNumber = true;
					}
				}
				all_values[dim.size()-1].push_back(tmp_vec);
			}
		}
		else {
			for(size_t i = 0; i < dim.size(); i++) {
				if (dim[0].size() > 0){
					tmp_vec.clear();
					tmp_vec.resize(dim[i][0]);
					y.clear();
					y.resize(dim[i][0]);
					for(size_t j = 0; j < dim[i].size(); j++) {
						for(size_t k = 0; k < dim[i][0]; k++) {
							if (all_values[i][j][k].isNumber){
								y[k].push_back(all_values[i][j][k].val);
							}
						}
					}
					for(size_t k = 0; k < dim[i][0]; k++) {
						sort(y[k].begin(), y[k].end());
					}
					for(size_t k = 0; k < dim[i][0]; k++) {
						if (y[k].size()%2 == 1){
							tmp_vec[k].val = y[k][y[k].size()/2];
							tmp_vec[k].isNumber = true;
						}
						else {
							tmp_vec[k].val = (y[k][(y[k].size()/2)-1] + y[k][y[k].size()/2])/2;
							tmp_vec[k].isNumber = true;
						}
					}
					all_values[i].push_back(tmp_vec);
				}
			}
		}
	}
	else {
		for(size_t i = 0; i < dim.size(); i++) {
			for(size_t j = 0; j < dim[i].size(); j++) {
				for(size_t k = 0; k < dim[i][j]; k++) {
					if (all_values[i][j][k].isNumber){
						x.push_back(all_values[i][j][k].val);
					}
				}
				sort(x.begin(), x.end());
				if (x.size()%2 == 1){
					tmp = x[x.size()/2];
				}
				else {
					tmp = (x[(x.size()/2)-1] + x[x.size()/2])/2; //when number of values is even median is average from 2 in the middle
				}
				all_values[i][j].push_back(Value(tmp));
				x.clear();
			}
		}
	}
}

template <typename T> void DataOperations<T>::printOut() {
	if (file_out.size() == 0) {
		printToStd();
	}
	else if (file_out.size() == 1 && (isMerged || isJoined || file_names.size() == 1)) {
		printToFile();
	}
	else if (file_names.size() == file_out.size()) {
		printToFiles();
	}
	else {
		int c = 1;
		while(c) {
			cout << "\nWARNING!!!Output and input file lists doesnt match to any standard pattern \nYou have three options:\n1. Print all to standard output.\n2. Print all to first file on the output file list.\n\n0. Exit without saving.\n(select 1, 2 or 0): ";
			cin >> c;
			switch (c) {
				case 1:
					printToStd();
					c = 0;
					break;
				case 2:
					printToFile();
					c = 0;
					break;
				case 0:
					break;
				default:
					break;
			}
		}
	}
}

template <typename T> void DataOperations<T>::commOut(ostream* out) {
	time_t rawtime;
	struct tm* timeinfo;
	string temp;
	if (isMerged) {
		temp = "# Data from files: ";
		for (size_t i = 0; i < file_names.size(); i++){
			temp += file_names[i];
			if (i < file_names.size()-1) temp += ",";
			temp += " ";
		}
	}
	if (commentMode >= 0){
		time(&rawtime);
		timeinfo = localtime(&rawtime);
		if (isJoined){
			*out << "#" << command << endl;
			*out << "#" << asctime(timeinfo) << endl;
			if (isMerged) *out << temp << endl;
			if (commentMode == 1){
				for(size_t i = 0; i < all_comments.size(); i++) {
					*out << all_comments[i][0] << endl;
				}
			}
			else if (commentMode == 2){
				for(size_t i = 0; i < all_comments.size(); i++) {
					for(size_t j = 0; j < all_comments[i].size(); j++) {
						*out << all_comments[i][j] << endl;
					}
				}
			}
		}
	}
}

template <typename T> void DataOperations<T>::commIn(ostream* out, int i) {
	time_t rawtime;
	struct tm* timeinfo;
	string temp;
	if (isMerged) {
		temp = "# Data from files: ";
		for (size_t i = 0; i < file_names.size(); i++){
			temp += file_names[i];
			if (i < file_names.size()-1) temp += ",";
			temp += " ";
		}
	}
	if (!isJoined && commentMode >= 0){
		time(&rawtime);
		timeinfo = localtime(&rawtime);
		if (!i){
			*out << "#" << command << endl;
			*out << "#" << asctime(timeinfo) << endl;
			if (isMerged) *out << temp << endl;
		}
		if (commentMode == 1){
			*out << all_comments[i][0] << endl;
		}
		else if (commentMode == 2){
			for(size_t j = 0; j < all_comments[i].size(); j++) {
				*out << all_comments[i][j] << endl;
			}
		}
	}
}

template <typename T> void DataOperations<T>::printToStd() {
	commOut(&cout);
	for(size_t i = 0; i < all_values.size(); i++) {
		if (all_values.size() < all_comments.size()){
			for (size_t f; f < all_comments.size(); f++){
				commIn(&cout, f);
			}
		}
		else {
			commIn(&cout, i);
		}
		for(size_t j = 0; j < all_values[i].size(); j++) {
			for(size_t k = 0; k < all_values[i][j].size(); k++) {
				if (all_values[i][j][k].isNumber) {
					if (precision > 0) {
						cout << setiosflags(ios::showpoint) << setprecision(precision) << all_values[i][j][k].val;
					}
					else {
						cout << all_values[i][j][k].val;
					}
				}
				else cout << blank;
				if (tabulator) cout << "\t";
				else cout << " ";
			}
			cout << endl;
		}
		if (!isJoined) cout << endl;
	}

}

template <typename T> void DataOperations<T>::printToFile() {
	ofstream outfile;
	outfile.open(file_out[0].c_str());
	commOut(&outfile);
	for(size_t i = 0; i < all_values.size(); i++) {
		if (all_values.size() < all_comments.size()){
			for (size_t f; f < all_comments.size(); f++){
				commIn(&outfile, f);
			}
		}
		else {
			commIn(&outfile, i);
		}
		for(size_t j = 0; j < all_values[i].size(); j++) {
			for(size_t k = 0; k < all_values[i][j].size(); k++) {
				if (all_values[i][j][k].isNumber) {
					if (precision > 0) {
						outfile << setiosflags(ios::showpoint) << setprecision(precision) << all_values[i][j][k].val;
					}
					else {
						outfile << all_values[i][j][k].val;
					}
				}
				else outfile << blank;
				if (tabulator) outfile << "\t";
				else outfile << " ";
			}
			outfile << endl;
		}
		if (!isJoined) outfile << endl;
	}
}

template <typename T> void DataOperations<T>::printToFiles() {
	ofstream outfile;
	for(size_t i = 0; i < all_values.size(); i++) {
		outfile.open(file_out[i].c_str());
		commIn(&outfile, i);
		for(size_t j = 0; j < all_values[i].size(); j++) {
			for(size_t k = 0; k < all_values[i][j].size(); k++) {
				if (all_values[i][j][k].isNumber) {
					if (precision > 0) {
						outfile << setiosflags(ios::showpoint) << setprecision(precision) << all_values[i][j][k].val;
					}
					else {
						outfile << all_values[i][j][k].val;
					}
				}
				else outfile << blank;
				if (tabulator) outfile << "\t";
				else outfile << " ";
			}
			outfile << endl;
		}
		outfile.close();
	}
}

template <typename T> void DataOperations<T>::setOutputFiles(string names) {
	char delim = ',';
	string name;
	istringstream iss(names);
	while (getline(iss, name, delim)) {
		file_out.push_back(name);
	}
}

/* Filtr działa w następujący sposób :
	dla opcji -f 1,gt12,0,1,2 oznacza to: wypisz wiersze w kolumnach 1,2 dla których wartości w pierwszej
	kolumnie jest większa od 12. 0 oznacza ze jest tylko jeden warunek (nie ma 'and' czy tez 'or')
	i oddziela kolumny do wypisania od warunku.
*/
template <typename T> void DataOperations<T>::setFilter(string arg) {
	int z, outt;
	double val1, val2;
	char *war1 = new char[2];
	char *war2 = new char[2];
	char *tmp = new char[100];
	char *tmp2 = new char[100];
	c = '0';
	int aa;
	const char *b = arg.c_str();
	z = sscanf(b, "%d,%2s%lf,%s", &aa, war1, &val1, tmp);
	col_cond = aa;
	if(z < 3 || (strcmp(war1, "lt") && strcmp(war1, "gt") && strcmp(war1, "le") && strcmp(war1, "ge") && strcmp(war1, "eq") && strcmp(war1, "ne"))) {
		cerr << "Wrong filtr argument" << endl;
		exit(1);
	}
	col_cond--;
	kind_of_cond.push_back(string(war1));
	conditions.push_back(val1);
	z = sscanf(tmp, "%c", &c);
	if(c != '0') {
		if(c == 'o' || c == 'a') {
			strcpy(tmp2, tmp);
			strcpy(tmp, "");
			z = sscanf(tmp2, "%*c,%2s%lf,%s", war2, &val2, tmp);
			if((z < 2 && !strcmp(tmp, "")) || (strcmp(war2, "lt") && strcmp(war2, "gt") && strcmp(war2, "le") && strcmp(war2, "ge") && strcmp(war2, "eq") && strcmp(war1, "ne"))) {
				cerr << "Wrong filtr argument" << endl;
				exit(1);
			}
			kind_of_cond.push_back(string(war2));
			conditions.push_back(val2);
		}
		else {
			cerr << "Wrong filtr argument" << endl;
			exit(1);
		}
	}
	if(strcmp(tmp, "")) {
		while(strcmp(tmp, "")) {
			strcpy(tmp2, tmp);
			strcpy(tmp, "");
			z = sscanf(tmp2, "%d,%s", &outt, tmp);
			if((z<1 && !strcmp(tmp, ""))) {
				cerr << "Wrong filtr argument" << endl;
				exit(1);
			}
			out.push_back(--outt);
		}
	}
	startFiltering();
}

template <typename T> size_t DataOperations<T>::startFiltering() {
	size_t a = 0;
	tab_tmp.clear();
	for(size_t i = 0; i < all_values.size(); i++) {
		for(size_t j = 0; j < all_values[i].size(); j++) {
			for(size_t k = 0; k < kind_of_cond.size(); k++) {
				if(all_values[i][j].size() <= col_cond || !all_values[i][j][col_cond].isNumber) {
					a = 0;
					break;
				}
				if(kind_of_cond[k].compare("eq") == 0) {
					if( fabs(all_values[i][j][col_cond].val-conditions[k]) <= conditions[k]+tolerance-conditions[k] ) a++;
					else a = 0;
				}
				else if(kind_of_cond[k].compare("lt") == 0) {
					if(all_values[i][j][col_cond].val < conditions[k]+tolerance) a++;
					else a = 0;
				}
				else if(kind_of_cond[k].compare("le") == 0) {
					T x = (T) conditions[k]+tolerance;
					if(all_values[i][j][col_cond].val <= x) a++;
					else a = 0;
				}
				else if(kind_of_cond[k].compare("gt") == 0) {
					if(all_values[i][j][col_cond].val > conditions[k]-tolerance) a++;
					else a = 0;
				}
				else if(kind_of_cond[k].compare("ge") == 0) {
					if(all_values[i][j][col_cond].val >= conditions[k]-tolerance) a++;
					else a = 0;
				}
				else if(kind_of_cond[k].compare("ne") == 0) {
					if( fabs(all_values[i][j][col_cond].val-conditions[k]) > tolerance) a++;
					else a = 0;
				}
				if(c == 'o') {
					if(a != 0) break;
				}//gdy or jeśli spełniony pierwszy warunek przerywamy dalsze sprawdzanie i wypisujemy
				else if(c == 'a') {
					if(a == 0) break;
				}//gdy and jeśli nie spełniony pierwszy warunek przerywamy dlasze sprawdzanie i nie wypisujemy
			}
			if(a != 0) {
				vall_tmp.clear();
				if(out.size() > 0) {
					for(size_t l = 0; l < out.size(); l++){
						if(all_values[i][j].size() > out[l]) {
							vall_tmp.push_back(all_values[i][j][out[l]]);
						}
					}
				}
				else {
					for(size_t l = 0; l < all_values[i][j].size(); l++){
						vall_tmp.push_back(all_values[i][j][l]);
					}
				}
				tab_tmp.push_back(vall_tmp);
			}
		}
	}
	all_values.clear();
	all_values.push_back(tab_tmp);
	return 0;
}

template <typename T> void DataOperations<T>::group(string arg) {
	size_t col;
	if (arg.compare("f") == 0){
		for(size_t i = 1; i < all_values.size(); i++) {
			for(size_t j = 0; j < all_values[i].size(); j++) {
				for(size_t k = 0; k < all_values[i][j].size(); k++) {
					all_values[0][j].push_back(all_values[i][j][k]);
				}
				all_values[i][j].clear();
			}
			all_values[i].clear();
		}
		all_values.resize(1);
	}
	else {
		col = atoi(arg.c_str());
		if (col > 0){
			col--;
			bool r = true;
			typedef map< T, vector<Value> > Mymap;
			Mymap m;
			for(size_t i = 0; i < all_values.size(); i++) {
				for(size_t j = 0; j < all_values[i].size(); j++) {
					if ( col < all_values[i][0].size() ){
						r = false;
						for(size_t k = 0; k < all_values[i][j].size(); k++) {
							if (k != col && all_values[i][j][col].isNumber){
								m[ all_values[i][j][col].val ].push_back(all_values[i][j][k]);
							}
						}
					}
				}
			}
			if (r) {
				cerr << "Wrong group argument" << endl;
				exit(1);
			}
			tab_tmp.clear();
			typename Mymap::iterator it;
			for (it = m.begin(); it != m.end(); it++){
				T i = (*it).first;
				m[i].insert(m[i].begin(), Value(i)); // goup column goes first
				tab_tmp.push_back(m[i]);
			}
			all_values.clear();
			all_values.push_back(tab_tmp);
			fillInBlanks();
		}
		else {
			cerr << "Wrong group argument" << endl;
			exit(1);
		}
	}
}

template <typename T> void DataOperations<T>::comments(string arg){
	if (arg.compare("off") == 0){
		commentMode = -1;
	}
	else if (arg.compare("all") == 0){
		commentMode = 2;
	}
	else if (arg.compare("1") == 0){
		commentMode = 1;
	}
	else {
		cerr << "Wrong comments argument" << endl;
		exit(1);
	}
}

template <typename T> void DataOperations<T>::calculate(string arg){
	if (arg.size() < 10 || arg.size() > 0){
		dim.clear();
		dim.resize(all_values.size());
		for(size_t i = 0; i < all_values.size(); i++) {
			dim[i].resize(all_values[i].size());
			for(size_t j = 0; j < all_values[i].size(); j++) {
				dim[i][j] = all_values[i][j].size();
			}
		}
		string calc(arg);
		while (calc.size()){
			switch (calc[0]) {
				case 'a':
					mean();
					break;
				case 'c':
					count();
					break;
				case 'd':
					deviation();
					break;
				case 'm':
					median();
					break;
				case 's':
					sum();
					break;
				default:
					cerr << "Wrong calculate argument" << endl;
					exit(1);
			}
			if (calc[1] == ','){
				calc = calc.substr(2);
			}
			else if (calc.size() != 1){
				cerr << "Wrong calculate argument" << endl;
				exit(1);
			}
			else{
				calc = "";
			}
		}
	}
	else {
		cerr << "Wrong calculate argument" << endl;
		exit(1);
	}
}

template <typename T> void DataOperations<T>::print(string arg){
	string prt(arg);
	string tmp;
	vector< vector<Value> > tmp_vec;
	vector< size_t > pa;
	vector<string> what;
	bool isParam = false;
	while (prt.size()){
		int k;
		if (prt.find_first_of(",") != string::npos){
			k = prt.find_first_of(",");
			if (k == 0){
				cerr << "Wrong print argument" << endl;
				exit(1);
			}
			tmp = prt.substr(0, k);
			what.push_back(tmp);
			prt = prt.substr(k+1);
		}
		else if (prt.size() > 0){
			what.push_back(prt);
			prt = "";
		}
		else {
			cerr << "Wrong print argument" << endl;
			exit(1);
		}
	}
	bool f_n = true;
	for (size_t w = 0; w < what.size(); w++){
		isParam = false;
		size_t col = atoi(what[w].c_str());
		if(col == 0 && (what[w] == "0" || what[w].find("0.0") == 0)) {
			cerr << "Wrong print argument" << endl;
			exit(1);
		}
		else if (col == 0) {
			for (size_t p = 0; p < all_param.size(); p++){
				if (all_param[p].count(what[w])) isParam = true;
			}
			if (!isParam){
				cerr << "Wrong print argument" << endl;
				exit(1);
			}
			else pa.push_back(w);
		}
		else {
			col--;
		}
		size_t l = 1;
		if (!isParam){
			for(size_t i = 0; i < all_values.size(); i++) {
				for(size_t j = 0; j < all_values[i].size(); j++) {
					if (f_n) tmp_vec.resize(l);
					if (col < all_values[i][j].size()){
						for(size_t k = 0; k < all_values[i][j].size(); k++) {
							if (k == col){
								tmp_vec[l-1].push_back(all_values[i][j][k]);
							}
						}
					}
					else {
						tmp_vec[l-1].push_back(Value());
					}
					l++;
				}
			}
			f_n = false;
		}
	}
	bool p_only = false;
	if (tmp_vec.size() == 0){
		tmp_vec.resize(all_values.size());
		p_only = true;
	}
	if (p_only){
		for (size_t w = 0; w < pa.size(); w++){
			size_t a_f = 0;
			for(size_t i = 0; i < tmp_vec.size(); i++){
				if (all_param[i].count( what[ pa[w] ] ) ){
					tmp_vec[i].push_back(Value( (T) all_param[a_f][what[ pa[w] ]]));
				}
				else {
					tmp_vec[i].push_back(Value());
				}
				a_f++;
			}
		}
	}
	else {
		for (size_t w = 0; w < pa.size(); w++){
			size_t a_f = 0;
			for(size_t i = 0; i < tmp_vec.size(); i++){ //lines
				size_t p = 0;
				for (size_t f = 0; f < a_f+1; f++) p += all_values[f].size();
				if (i == p) a_f++;
				typename vector<Value>::iterator it;
				for(size_t j = 0; j < tmp_vec[i].size(); j++){
					it = tmp_vec[i].begin();
					for(size_t k = 0; k < j; k++) it++;
					if (pa[w] == j){
						if (all_param[a_f].count(what[ pa[w] ])){
							tmp_vec[i].insert(it, Value( (T) all_param[a_f][what[ pa[w] ]]));
						}
						else {
							tmp_vec[i].insert(it, Value());
						}
					}
					else if (pa[w] == tmp_vec[i].size()){
						if (all_param[a_f].count(what[ pa[w] ])){
							tmp_vec[i].push_back(Value( (T) all_param[a_f][what[ pa[w] ]]));
						}
						else {
							tmp_vec[i].push_back(Value());
						}
						break;
					}
				}
			}
		}
	}

	all_values.clear();
	all_values.push_back(tmp_vec);
}

template <typename T> void DataOperations<T>::merge(){
	isMerged = true;
	for(size_t i = 0; i < all_values.size(); i++) {
		for(size_t j = 0; j < all_values[i].size(); j++) {
			all_values[i][j].insert(all_values[i][j].begin(), Value(i+1));
		}
	}
}
